import {
  createReactFlowElements,
  DragAttributePill,
  DragAttributesAlong,
} from '@graphpolaris/querybuilder/usecases';
import {
  setQuerybuilderNodes,
  useAppDispatch,
  useQuerybuilderNodes,
} from '@graphpolaris/shared/data-access/store';
import {
  AllLayoutAlgorithms,
  LayoutFactory,
} from '@graphpolaris/shared/graph-layout';
import Graph, { MultiGraph } from 'graphology';
import { useEffect, useMemo, useState } from 'react';
import ReactFlow, {
  ReactFlowProvider,
  FlowElement,
  Background,
  Node,
  isNode,
} from 'react-flow-renderer';
import styles from './querybuilder.module.scss';
import ConnectionLine from './customFlowLines/connection';
import ConnectionDragLine from './customFlowLines/connectionDrag';
import AttributeRFPill from './customFlowPills/attributepill/attributepill';
import EntityRFPill from './customFlowPills/entitypill/entitypill';
import RelationRFPill from './customFlowPills/relationpill/relationpill';

const nodeTypes = {
  entity: EntityRFPill,
  relation: RelationRFPill,
  attribute: AttributeRFPill,
};
const edgeTypes = {
  connection: ConnectionLine,
};

const onLoad = (reactFlowInstance: any) => {
  setTimeout(() => reactFlowInstance.fitView(), 0);
};

const QueryBuilder = (props: {}) => {
  const nodes = useQuerybuilderNodes();
  const dispatch = useAppDispatch();

  const elements = useMemo(() => createReactFlowElements(nodes), [nodes]);

  const onNodeDrag = (
    event: React.MouseEvent<Element, MouseEvent>,
    node: Node<any>
  ) => {
    const pNode = elements.find((e) => e.id == node.id);
    if (!(pNode && isNode(pNode))) return;

    const dx = node.position.x - pNode.position.x;
    const dy = node.position.y - pNode.position.y;

    nodes.setNodeAttribute(node.id, 'x', node.position.x);
    nodes.setNodeAttribute(node.id, 'y', node.position.y);

    switch (nodes.getNodeAttribute(node.id, 'type')) {
      case 'attribute':
        DragAttributePill(node.id, nodes, dx, dy);
        break;
      case 'entity':
        DragAttributesAlong(node.id, nodes, dx, dy);
        break;
      case 'relation':
        DragAttributesAlong(node.id, nodes, dx, dy);
        break;
    }

    dispatch(setQuerybuilderNodes(nodes.export()));
  };

  return (
    <div>
      <ReactFlowProvider>
        <ReactFlow
          elements={elements}
          snapGrid={[10, 10]}
          // snapToGrid
          nodeTypes={nodeTypes}
          edgeTypes={edgeTypes}
          connectionLineComponent={ConnectionDragLine}
          // onLoad={onLoad}
          onNodeDrag={onNodeDrag}
          className={styles.reactflow}
        >
          <Background gap={10} size={0.7} />
        </ReactFlow>
      </ReactFlowProvider>
    </div>
  );
};

export default QueryBuilder;
